# -*- coding: utf-8 -*-

from Acquisition import aq_base
from plone.app.dexterity.behaviors.metadata import ICategorization
from plone.app.dexterity.textindexer.utils import searchable
from plone.resource.file import FilesystemFile
from Products.CMFPlone.resources import utils
from Products.CMFPlone.resources import webresource
from Products.CMFPlone.resources.utils import get_override_directory
from Products.CMFPlone.resources.utils import logger
from zExceptions import NotFound

searchable(ICategorization, "subjects")


def patched_get_resource(context, path):
    """
    Avoid errors with TTW theme resources
    See https://github.com/plone/Products.CMFPlone/issues/3705
    """
    if path.startswith("++plone++"):
        # ++plone++ resources can be customized, we return their override
        # value if any
        overrides = get_override_directory(context)
        filepath = path[9:]
        if overrides.isFile(filepath):
            return overrides.readFile(filepath)

    if "?" in path:
        # Example from plone.session:
        # "acl_users/session/refresh?session_refresh=true&type=css&minutes=5"
        # Traversing will not work then.  In this example we could split on "?"
        # and traverse to the first part, acl_users/session/refresh, but this
        # gives a function, and this fails when we call it below, missing a
        # REQUEST argument
        return
    try:
        resource = context.unrestrictedTraverse(path)
    except (NotFound, AttributeError, KeyError):
        logger.warning(
            f"Could not find resource {path}. You may have to create it first."
        )  # noqa
        return

    if isinstance(resource, FilesystemFile):
        (directory, sep, filename) = path.rpartition("/")
        return context.unrestrictedTraverse(directory).readFile(filename)

    # calling the resource may modify the header, i.e. the content-type.
    # we do not want this, so keep the original header intact.
    response_before = context.REQUEST.response
    context.REQUEST.response = response_before.__class__()
    if hasattr(aq_base(resource), "GET"):
        # for FileResource
        result = resource.GET()
    else:
        # any BrowserView
        try:
            # MONKEY IMIO: this avoids AttributeError: __call__ on resources
            result = resource()
        except AttributeError:
            if isinstance(resource.data, bytes):
                result = resource.data
            elif isinstance(resource.data.data, bytes):
                result = resource.data.data
    context.REQUEST.response = response_before
    return result


utils.get_resource = patched_get_resource
webresource.get_resource = patched_get_resource



# -*- coding: utf-8 -*-

from collective.privacy.browser.consent import ConsentForm
from imio.smartweb.common.browser.privacy.utils import get_all_consent_reasons
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from z3c.form import button
from zope.i18n import translate


class ConsentFormWithPolicy(ConsentForm):
    label = _("Cookies choice")
    id = "cookies-form"

    def update(self):
        super(ConsentFormWithPolicy, self).update()
        root = api.portal.get_navigation_root(self.context)
        current_lang = api.portal.get_current_language()[:2]
        policy_url = "{}/@@cookies-view".format(root.absolute_url())
        description = _(
            "Choose to opt in or out of cookies use.<br/>"
            'Our <a href="${policy_url}">cookies policy</a> can help you choose.',
            mapping={"policy_url": policy_url},
        )
        self.description = translate(description, target_language=current_lang)

    @button.buttonAndHandler(_("Save my choices"))
    def handleApply(self, action):
        super(ConsentFormWithPolicy, self).handleApply(self, action)

    @button.buttonAndHandler(_("Accept all"))
    def handleAcceptAll(self, action):
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            privacy_tool.consentToProcessing(reason.__name__)

    @button.buttonAndHandler(_("Refuse all"))
    def handleRefuseAll(self, action):
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            privacy_tool.objectToProcessing(reason.__name__)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
)
from plone import api
from plone.testing import z2

import imio.smartweb.common
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
import unittest


class ImioSmartwebCommonLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=imio.smartweb.common)

    def setUpPloneSite(self, portal):
        api.user.create(email="test@imio.be", username="test")
        applyProfile(portal, "imio.smartweb.common:testing")


IMIO_SMARTWEB_COMMON_FIXTURE = ImioSmartwebCommonLayer()


IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING = IntegrationTesting(
    bases=(IMIO_SMARTWEB_COMMON_FIXTURE,),
    name="ImioSmartwebCommonLayer:IntegrationTesting",
)


IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(IMIO_SMARTWEB_COMMON_FIXTURE,),
    name="ImioSmartwebCommonLayer:FunctionalTesting",
)


IMIO_SMARTWEB_COMMON_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        IMIO_SMARTWEB_COMMON_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="ImioSmartwebCommonLayer:AcceptanceTesting",
)


class ImioSmartwebCommonTestCase(unittest.TestCase):
    def assertVocabularyLen(self, vocname, voc_len):
        factory = getUtility(IVocabularyFactory, vocname)
        vocabulary = factory()
        self.assertEqual(len(vocabulary), voc_len)



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ICropping
from operator import itemgetter
from plone.app.imagecropping.browser.editor import CroppingEditor
from plone.namedfile.interfaces import IAvailableSizes
from zope.component import getUtility

import six


class SmartwebCroppingEditor(CroppingEditor):
    def _scales(self, fieldname):
        adapter = ICropping(self.context, alternate=None)
        if adapter is None:
            yield from super(SmartwebCroppingEditor, self)._scales(fieldname)
            return
        allowed_sizes = getUtility(IAvailableSizes)() or []
        sizes_iterator = sorted(six.iteritems(allowed_sizes), key=itemgetter(1))
        context_scales = adapter.get_scales(fieldname, self.request)
        for scale_id, target_size in sizes_iterator:
            if scale_id not in context_scales:
                continue
            yield scale_id, target_size



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.i18n.locales import locales
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import json


class TopicsVocabularyFactory:
    def __call__(self, context=None):
        topics = [
            ("entertainment", _("Entertainment")),
            ("agriculture", _("Agriculture")),
            ("citizenship", _("Citizenship")),
            ("culture", _("Culture")),
            ("economics", _("Economics")),
            ("education", _("Education")),
            ("environment", _("Environment")),
            ("habitat_town_planning", _("Habitat and town planning")),
            ("mobility", _("Mobility")),
            ("citizen_participation", _("Citizen participation")),
            ("politics", _("Politics")),
            ("health", _("Health")),
            ("safety_prevention", _("Safety and prevention")),
            ("social", _("Social")),
            ("sports", _("Sports")),
            ("territory_public_space", _("Territory and public space")),
            ("tourism", _("Tourism")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in topics]
        return SimpleVocabulary(terms)


TopicsVocabulary = TopicsVocabularyFactory()


class IAmVocabularyFactory:
    def __call__(self, context=None):
        iam = [
            ("merchant", _("Merchant")),
            ("job_seeker", _("Job seeker")),
            ("disabled_person", _("Disabled person")),
            ("young", _("Young")),
            ("journalist", _("Journalist")),
            ("newcomer", _("Newcomer")),
            ("event_planner", _("Event planner")),
            ("parent", _("Parent")),
            ("elder", _("Elder")),
            ("tourist", _("Tourist")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in iam]
        return SimpleVocabulary(terms)


IAmVocabulary = IAmVocabularyFactory()


class CountriesVocabularyFactory:
    def __call__(self, context=None, lang=None):
        normalizer = getUtility(IIDNormalizer)
        if lang is None:
            lang = api.portal.get_current_language()
        locale = locales.getLocale(lang)
        localized_country_names = {
            capitalized_code.lower(): translation
            for capitalized_code, translation in locale.displayNames.territories.items()
        }
        terms = [
            SimpleTerm(value=k, token=k, title=v)
            for k, v in sorted(
                localized_country_names.items(),
                key=lambda kv: normalizer.normalize(kv[1]),
            )
            if k != "fallback"
        ]
        return SimpleVocabulary(terms)


CountriesVocabulary = CountriesVocabularyFactory()


class CitiesVocabularyFactory:
    def __call__(self, context=None):
        registry = getUtility(IRegistry)
        json_str = registry.get("imio.smartweb.cities")
        cities = json.loads(json_str)
        terms = [
            SimpleVocabulary.createTerm(
                city["zip"], city["zip"], "{0} {1}".format(city["zip"], city["city"])
            )
            for city in cities
        ]
        return SimpleVocabulary(terms)


CitiesVocabulary = CitiesVocabularyFactory()



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import IAddress
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from imio.smartweb.common.utils import geocode_object
from imio.smartweb.common.utils import get_term_from_vocabulary
from imio.smartweb.common.utils import get_uncroppable_scales_infos
from imio.smartweb.common.utils import show_warning_for_scales
from imio.smartweb.common.utils import translate_vocabulary_term
from imio.smartweb.common.utils import clean_invisible_char
from plone import api
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.namedfile.file import NamedBlobImage
from Products.statusmessages.interfaces import IStatusMessage
from unittest import mock
from unittest.mock import patch
from zope.interface import implementer

import geopy
import os
import unittest


@implementer(IAddress)
class GeolocatedObject(object):
    """Dummy class for geolocation tests"""

    street = None
    number = None
    complement = None
    zipcode = None
    city = None
    country = None

    def reindexObject(self, idxs):
        return


class TestUtils(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_get_term_from_vocabulary(self):
        term = get_term_from_vocabulary("imio.smartweb.vocabulary.Topics", "culture")
        self.assertEqual(term.title, "Culture")

        term = get_term_from_vocabulary("imio.smartweb.vocabulary.IAm", "parent")
        self.assertEqual(term.title, "Parent")

        term = get_term_from_vocabulary("imio.smartweb.vocabulary.IAm", "non_existing")
        self.assertEqual(term.title, "non_existing")

    def test_translate_vocabulary_term(self):
        self.assertEqual(
            translate_vocabulary_term("imio.smartweb.vocabulary.Countries", None),
            "",
        )
        self.assertEqual(
            translate_vocabulary_term("imio.smartweb.vocabulary.Countries", "be"),
            "Belgium",
        )
        self.assertEqual(
            translate_vocabulary_term("imio.smartweb.vocabulary.Countries", "be", "fr"),
            "Belgique",
        )
        self.assertEqual(
            translate_vocabulary_term("imio.smartweb.vocabulary.Countries", "be", "nl"),
            "België",
        )
        with patch("plone.api.portal.get_current_language", return_value="fr"):
            self.assertEqual(
                translate_vocabulary_term("imio.smartweb.vocabulary.Countries", "be"),
                "Belgique",
            )

    def test_geolocation(self):
        attr = {"geocode.return_value": mock.Mock(latitude=1, longitude=2)}
        geopy.geocoders.Nominatim = mock.Mock(return_value=mock.Mock(**attr))

        obj = GeolocatedObject()
        obj.geolocation = Geolocation(0, 0)

        geocoded = geocode_object(obj)
        self.assertFalse(geocoded)
        self.assertEqual(obj.geolocation.latitude, 0)
        self.assertEqual(obj.geolocation.longitude, 0)

        obj.street = "My beautiful street"
        geocoded = geocode_object(obj)
        self.assertTrue(geocoded)
        self.assertEqual(obj.geolocation.latitude, 1)
        self.assertEqual(obj.geolocation.longitude, 2)

    def test_get_uncroppable_scales_infos(self):
        folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
            id="folder",
        )
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 0)

        test_image = os.path.join(
            os.path.dirname(__file__), "resources/image_1800x700.png"
        )
        with open(test_image, "rb") as fd:
            folder.image = NamedBlobImage(data=fd.read(), filename=test_image)

        sizes = {"size1": (1000, 500), "size2": (65536, 500)}
        scales = []
        result = get_uncroppable_scales_infos(folder.image, sizes, scales)
        self.assertEqual(result, {})

        sizes = {"size1": (1000, 500), "size2": (65536, 500)}
        scales = ["size1", "size2"]
        result = get_uncroppable_scales_infos(folder.image, sizes, scales)
        self.assertEqual(result, {})

        sizes = {"size1": (1900, 500), "size2": (65536, 500)}
        scales = ["size1", "size2"]
        result = get_uncroppable_scales_infos(folder.image, sizes, scales)
        self.assertEqual(len(result["scales"]), 1)
        self.assertEqual(result["min_width"], "1900")
        self.assertEqual(result["min_height"], "500")
        self.assertEqual(result["width"], "1800")
        self.assertEqual(result["height"], "700")

        sizes = {"size1": (1900, 400), "size2": (1700, 600), "size3": (65536, 800)}
        scales = ["size1", "size2", "size3"]
        result = get_uncroppable_scales_infos(folder.image, sizes, scales)
        self.assertEqual(len(result["scales"]), 2)
        self.assertEqual(result["min_width"], "1900")
        self.assertEqual(result["min_height"], "800")

    def test_show_warning_for_scales(self):
        folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
            id="folder",
        )
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 0)

        test_image = os.path.join(os.path.dirname(__file__), "resources/image.png")
        with open(test_image, "rb") as fd:
            folder.image = NamedBlobImage(data=fd.read(), filename=test_image)
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 1)
        self.assertEqual(
            show[0].message,
            'The image uploaded in the "Lead Image" field may be degraded because '
            "it does not meet the required minimum dimensions of 1320px width by 768px height "
            "(uploaded image size: 215px width by 56px height). "
            "You can see the detail via the Cropping menu.",
        )

        test_image = os.path.join(
            os.path.dirname(__file__), "resources/image_1800x700.png"
        )
        with open(test_image, "rb") as fd:
            folder.image = NamedBlobImage(data=fd.read(), filename=test_image)
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 1)
        self.assertEqual(
            show[0].message,
            'The image uploaded in the "Lead Image" field may be degraded because '
            "it does not meet the required minimum dimensions of 1320px width by 768px height "
            "(uploaded image size: 1800px width by 700px height). "
            "You can see the detail via the Cropping menu.",
        )

        test_image = os.path.join(
            os.path.dirname(__file__), "resources/image_1400x800.png"
        )
        with open(test_image, "rb") as fd:
            folder.image = NamedBlobImage(data=fd.read(), filename=test_image)
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 0)

        test_image = os.path.join(os.path.dirname(__file__), "resources/image.png")
        with open(test_image, "rb") as fd:
            folder.image = NamedBlobImage(data=fd.read(), filename=test_image)
        logout()
        show_warning_for_scales(folder, self.request)
        messages = IStatusMessage(self.request)
        show = messages.show()
        self.assertEqual(len(show), 0)

    def test_clean_invisible_char(self):
        txt = "<p>Kam\x02oulox</p>"
        clean_txt = clean_invisible_char(txt)
        self.assertEqual(clean_txt, "<p>Kamoulox</p>")

        clean_txt = clean_invisible_char(None)
        self.assertIsNone(clean_txt)



# -*- coding: utf-8 -*-

from plone import api
import logging

logger = logging.getLogger("imio.smartweb.common")
PROFILEID = "profile-imio.smartweb.common:default"


def reload_registry(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "plone.app.registry")


def reload_actions(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "actions")


def reindex_searchable_text(context):
    portal_catalog = api.portal.get_tool("portal_catalog")
    portal_catalog.manage_reindexIndex(ids=["SearchableText"])
    logger.info("Reindexed SearchableText index")


def upgrade_barceloneta(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.upgradeProfile("plonetheme.barceloneta:default")



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ILocalManagerAware
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.workflow.permissions import DelegateRoles
from plone.testing.zope import Browser
from Products.CMFCore.utils import _checkPermission
from zope.interface import alsoProvides

import transaction
import unittest


class TestLocalRoles(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
        )

    def test_sharing_page_available(self):
        alsoProvides(self.folder, ILocalManagerAware)
        setRoles(self.portal, TEST_USER_ID, ["Editor"])
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertIn("Insufficient Privileges", content)

        self.folder.manage_setLocalRoles(TEST_USER_ID, ("Local Manager",))
        transaction.commit()
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertIn("Can add", content)
        self.assertIn("Can edit", content)
        self.assertIn("Can manage locally", content)
        self.assertIn("Can review", content)
        self.assertIn("Can view", content)
        self.assertNotIn("Insufficient Privileges", content)

        transaction.commit()
        browser.open("{}/@@sharing".format(self.portal.absolute_url()))
        content = browser.contents
        self.assertIn("Insufficient Privileges", content)

    def test_delegate_sharing_permission(self):
        self.assertTrue(_checkPermission(DelegateRoles, self.folder))
        setRoles(self.portal, TEST_USER_ID, ["Editor"])
        self.assertFalse(_checkPermission(DelegateRoles, self.folder))
        setRoles(self.portal, TEST_USER_ID, ["Local Manager"])
        self.assertTrue(_checkPermission(DelegateRoles, self.folder))

    def test_local_manager_in_sharing(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertNotIn("Can manage locally", content)

        alsoProvides(self.folder, ILocalManagerAware)
        transaction.commit()
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertIn("Can manage locally", content)



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import rich_description
from Products.CMFPlone.utils import base_hasattr
from Products.Five.browser import BrowserView


class RichDescription(BrowserView):
    def description(self):
        """Description with html carriage return and bold"""
        description = ""
        if base_hasattr(self.context, "description"):
            description = self.context.description or ""
        description = rich_description(description)
        return description



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import VOCABULARIES_MAPPING
from imio.smartweb.common.utils import get_term_from_vocabulary
from operator import itemgetter
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from plone.restapi.interfaces import ISerializeToJson
from zope.component import getMultiAdapter


# we don't need some brains metadata to construct search filters
EXCLUDED_METADATA = [
    "effective",
    "end",
    "has_leadimage",
    "start",
    "UID",
]


class SearchFiltersGet(Service):
    def reply(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        return SearchFiltersHandler(self.context, self.request).search(query)


class SearchFiltersHandler(SearchHandler):
    """
    Executes a catalog search based on a query dict, and returns the metadata
    values as vocabularies terms in JSON. This is used for example for smartweb
    faceted searches.
    """

    ignored_params = ["fullobjects", "b_size", "b_start"]

    def search(self, query=None):
        if query is None:
            query = {}

        for param in self.ignored_params:
            if param in query:
                del query[param]

        use_site_search_settings = False
        if "use_site_search_settings" in query:
            use_site_search_settings = True
            del query["use_site_search_settings"]

        if use_site_search_settings:
            query = self.filter_query(query)

        self._constrain_query_by_path(query)
        query = self._parse_query(query)

        if "metadata_fields" not in query:
            return {}

        brains = self.catalog.searchResults(**query)

        metadatas = query["metadata_fields"]
        if not isinstance(metadatas, list):
            metadatas = [metadatas]

        metadatas = list(set(metadatas) - set(EXCLUDED_METADATA))
        filters = {metadata: set() for metadata in metadatas}
        for brain in brains:
            for metadata in metadatas:
                value = getattr(brain, metadata, None)
                if not value:
                    continue
                if isinstance(value, list) or isinstance(value, tuple):
                    for v in value:
                        filters[metadata].add(v)
                else:
                    filters[metadata].add(value)

        results = {}
        for metadata, values in filters.items():
            results[metadata] = []
            if metadata in VOCABULARIES_MAPPING:
                for v in values:
                    term = get_term_from_vocabulary(VOCABULARIES_MAPPING[metadata], v)
                    title = term.title
                    serializer = getMultiAdapter(
                        (term, self.request), interface=ISerializeToJson
                    )
                    term = serializer()
                    term["title"] = json_compatible(title)  # needed for translations
                    results[metadata].append(term)
                    results[metadata].sort(key=itemgetter("title"))
            else:
                results[metadata] = [{"token": v, "title": v} for v in sorted(values)]

        return results



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue
from zope.lifecycleevent import modified

import unittest


class TestText(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="My Root Folder",
        )

    def test_cleaning_text(self):
        document = api.content.create(
            container=self.folder,
            type="Document",
            title="My Doc",
        )
        text = RichTextValue("<p>Kam\x02oulox</p>", "text/html", "text/html")
        document.text = text
        modified(document)
        self.assertEqual(document.text.raw, "<p>Kamoulox</p>")

        document = api.content.create(
            container=self.folder,
            type="Document",
            title="My Doc",
            text=RichTextValue("<p>Kam\x02oulox</p>", "text/html", "text/html"),
        )
        self.assertEqual(document.text.raw, "<p>Kamoulox</p>")



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.vocabularies.types import BAD_TYPES
from plone.uuid.interfaces import IUUID
from zope.component import queryMultiAdapter

import unittest


class TestDescription(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_description(self):
        portal_types = api.portal.get_tool("portal_types")
        bad_types = BAD_TYPES + ["Discussion Item"]
        all_types = [t for t in portal_types.listContentTypes() if t not in bad_types]
        for pt in all_types:
            content_type = api.content.create(
                title="My {}".format(pt), container=self.portal, type=pt
            )
            content_type.description = (
                "My bold **description** is wonderfull with \r\n carriage return \r\n"
            )
            view = queryMultiAdapter((content_type, self.request), name="description")
            self.assertEqual(
                view.description(),
                "My bold <strong>description</strong> is wonderfull with <br/> carriage return <br/>",
            )
            content_type.reindexObject()

            uuid = IUUID(content_type)
            brain = api.content.find(UID=uuid)[0]
            catalog = api.portal.get_tool("portal_catalog")
            indexes = catalog.getIndexDataForRID(brain.getRID())
            self.assertEqual(
                indexes.get("Description"),
                [
                    "my",
                    "bold",
                    "description",
                    "is",
                    "wonderfull",
                    "with",
                    "carriage",
                    "return",
                ],
            )

            content_type.description = "My bold **description** is wonderfull with ** \r\n carriage return \r\n"
            view = queryMultiAdapter((content_type, self.request), name="description")
            self.assertEqual(
                view.description(),
                "My bold <strong>description</strong> is wonderfull with ** <br/> carriage return <br/>",
            )



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import TRANSLATED_VOCABULARIES
from imio.smartweb.common.interfaces import ICropping
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.dexterity.utils import iterSchemata
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.namedfile.field import NamedBlobImage
from plone.namedfile.interfaces import IAvailableSizes
from zope.component import getUtility
from zope.i18n import translate
from zope.schema import getFields
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm

import geopy
import re
import unicodedata


def get_vocabulary(voc_name):
    factory = getUtility(IVocabularyFactory, voc_name)
    vocabulary = factory(api.portal.get())
    return vocabulary


def get_term_from_vocabulary(vocabulary, value):
    portal = api.portal.get()
    factory = getUtility(IVocabularyFactory, vocabulary)
    vocabulary = factory(portal)
    try:
        term = vocabulary.getTerm(value)
    except LookupError:
        return SimpleTerm(value=value, title=value)
    return term


def translate_vocabulary_term(vocabulary, term, lang=None):
    if term is None:
        return ""
    portal = api.portal.get()
    factory = getUtility(IVocabularyFactory, vocabulary)
    if vocabulary in TRANSLATED_VOCABULARIES:
        vocabulary = factory(portal, lang=lang)
    else:
        vocabulary = factory(portal)
    term = vocabulary.getTerm(term)
    if term is None:
        return ""
    if lang is None:
        lang = api.portal.get_current_language()[:2]
    return translate(term.title, target_language=lang)


def geocode_object(obj):
    street_parts = [
        obj.number and str(obj.number) or "",
        obj.street,
        obj.complement,
    ]
    street = " ".join(filter(None, street_parts))
    entity_parts = [
        obj.zipcode and str(obj.zipcode) or "",
        obj.city,
    ]
    entity = " ".join(filter(None, entity_parts))
    country = translate_vocabulary_term(
        "imio.smartweb.vocabulary.Countries", obj.country
    )
    address = " ".join(filter(None, [street, entity, country]))
    if not address:
        return
    geolocator = geopy.geocoders.Nominatim(user_agent="contact@imio.be", timeout=3)
    location = geolocator.geocode(address)
    if location:
        obj.geolocation = Geolocation(
            latitude=location.latitude, longitude=location.longitude
        )
        obj.reindexObject(idxs=["longitude", "latitude"])
        return True
    return False


def rich_description(description):
    # **strong**
    description = re.sub(r"\*\*([^\*\*]*)\*\*", r"<strong>\1</strong>", description)
    # <br/>
    description = "<br/>".join(description.split("\r\n"))
    return description


def get_uncroppable_scales_infos(image, available_sizes, scales):
    img_height = image._height
    img_width = image._width
    result = {}
    uncroppable_scales = set()
    min_height = min_width = 0
    for scale in scales:
        width, height = available_sizes.get(scale)
        if 65536 > width > img_width or 65536 > height > img_height:
            uncroppable_scales.add(scale)
        if 65536 > width > min_width:
            min_width = width
        if 65536 > height > min_height:
            min_height = height
    if uncroppable_scales:
        result = {
            "scales": sorted(list(uncroppable_scales)),
            "min_height": str(min_height),
            "min_width": str(min_width),
            "height": str(img_height),
            "width": str(img_width),
        }
    return result


def show_warning_for_scales(obj, request):
    if not api.user.has_permission("Modify portal content", obj=obj):
        return
    available_sizes = getUtility(IAvailableSizes)() or {}
    cropping_scales_adapter = ICropping(obj, alternate=None)
    for schema in iterSchemata(obj):
        for name, field in getFields(schema).items():
            if type(field) is not NamedBlobImage or not getattr(obj, name, None):
                continue
            field_scales = cropping_scales_adapter.get_scales(name, request)
            uncroppable_infos = get_uncroppable_scales_infos(
                getattr(obj, name), available_sizes, field_scales
            )
            if not uncroppable_infos:
                continue
            api.portal.show_message(
                _(
                    'The image uploaded in the "${field_title}" field may be '
                    "degraded because it does not meet the required minimum dimensions of "
                    "${min_width}px width by ${min_height}px height "
                    "(uploaded image size: ${width}px width by ${height}px height). "
                    "You can see the detail via the Cropping menu.",
                    mapping={
                        "field_title": field.title,
                        "min_height": uncroppable_infos["min_height"],
                        "min_width": uncroppable_infos["min_width"],
                        "height": uncroppable_infos["height"],
                        "width": uncroppable_infos["width"],
                    },
                ),
                request=request,
                type="warning",
            )


def clean_invisible_char(value):
    # surpassing all control characters
    # checking for starting with C
    if value is None:
        return value
    res = "".join(char for char in value if unicodedata.category(char)[0] != "C")
    return res



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.browser.add import DefaultAddView
from plone.dexterity.browser.edit import DefaultEditForm
from plone.z3cform import layout
from z3c.form.interfaces import HIDDEN_MODE


class CustomAddForm(DefaultAddForm):
    css_class = "tabbed-form-with-toggle"
    enable_form_tabbing = False

    def updateFields(self):
        super(CustomAddForm, self).updateFields()
        if "ILeadImageBehavior.image_caption" in self.fields:
            # We don't use leadimage caption anywhere
            self.fields["ILeadImageBehavior.image_caption"].mode = HIDDEN_MODE
        # Description field has a maximum number of chars
        if "IBasic.description" in self.fields:
            self.fields["IBasic.description"].field.max_length = DESCRIPTION_MAX_LENGTH
        elif "IDublinCore.description" in self.fields:
            self.fields[
                "IDublinCore.description"
            ].field.max_length = DESCRIPTION_MAX_LENGTH

    def updateWidgets(self):
        super(CustomAddForm, self).updateWidgets()
        if "ILeadImageBehavior.image" in self.widgets:
            desc = _(
                "Please note that it is important to upload an image without "
                "text for design and accessibility reasons. "
                "The image will not display in its entirety and the text may be cut off."
            )
            self.widgets["ILeadImageBehavior.image"].description = desc
        # Change Description field help text
        if "IBasic.description" in self.widgets:
            self.widgets["IBasic.description"].description = _(
                "Use **text** to set text in bold. Limited to ${max} characters.",
                mapping={"max": DESCRIPTION_MAX_LENGTH},
            )
        elif "IDublinCore.description" in self.widgets:
            self.widgets["IDublinCore.description"].description = _(
                "Use **text** to set text in bold. Limited to ${max} characters.",
                mapping={"max": DESCRIPTION_MAX_LENGTH},
            )


class CustomAddView(DefaultAddView):
    form = CustomAddForm


class CustomEditForm(DefaultEditForm):
    css_class = "tabbed-form-with-toggle"
    enable_form_tabbing = False

    def updateFields(self):
        super(CustomEditForm, self).updateFields()
        if "ILeadImageBehavior.image_caption" in self.fields:
            # We don't use leadimage caption anywhere
            self.fields["ILeadImageBehavior.image_caption"].mode = HIDDEN_MODE
        # Description field has a maximum number of chars
        if "IBasic.description" in self.fields:
            self.fields["IBasic.description"].field.max_length = DESCRIPTION_MAX_LENGTH
        elif "IDublinCore.description" in self.fields:
            self.fields[
                "IDublinCore.description"
            ].field.max_length = DESCRIPTION_MAX_LENGTH

    def updateWidgets(self):
        super(CustomEditForm, self).updateWidgets()
        if "ILeadImageBehavior.image" in self.widgets:
            desc = _(
                "Please note that it is important to upload an image without "
                "text for design and accessibility reasons. "
                "The image will not display in its entirety and the text may be cut off."
            )
            self.widgets["ILeadImageBehavior.image"].description = desc
        # Change Description field help text
        if "IBasic.description" in self.widgets:
            self.widgets["IBasic.description"].description = _(
                "Use **text** to set text in bold. Limited to ${max} characters.",
                mapping={"max": DESCRIPTION_MAX_LENGTH},
            )
        elif "IDublinCore.description" in self.widgets:
            self.widgets["IDublinCore.description"].description = _(
                "Use **text** to set text in bold. Limited to ${max} characters.",
                mapping={"max": DESCRIPTION_MAX_LENGTH},
            )


CustomEditView = layout.wrap_form(CustomEditForm)



# -*- coding: utf-8 -*-
from AccessControl import ModuleSecurityInfo
from AccessControl.Permission import addPermission

security = ModuleSecurityInfo("imio.smartweb.common.sharing.permissions")

# Control the individual roles
security.declarePublic("DelegateLocalManagerRole")
DelegateLocalManagerRole = "Sharing page: Delegate Local Manager role"
addPermission(
    DelegateLocalManagerRole,
    ("Manager", "Site Administrator", "Local Manager"),
)



# -*- coding: utf-8 -*-

from email.utils import formatdate
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.testing.z2 import Browser
from plone.transformchain.interfaces import ITransform
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter

import time
import unittest


class TestPrivacy(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_iframe_transform(self):
        self.request.response["content-type"] = "text/html"
        privacy_handler = getMultiAdapter(
            (True, self.request), ITransform, name="imio.smartweb.common.privacy"
        )
        result = privacy_handler.transformUnicode(
            '<iframe src="http://host.com">', "utf-8"
        )
        self.assertIn('src=""', result)
        self.assertIn('gdpr-src="http://host.com"', result)
        self.assertIn('class="gdpr-iframe"', result)

        html = '<iframe src="http://host.com" width="2" height="4">'
        result = privacy_handler.transformUnicode(html, "utf-8")
        self.assertIn('width="0"', result)
        self.assertIn('height="0"', result)
        self.assertIn('gdpr-width="2"', result)
        self.assertIn('gdpr-height="4"', result)
        self.assertIn("This feature requires cookies acceptation", result)

        self.assertEqual(privacy_handler.transformBytes(html, "utf-8"), result)
        self.assertEqual(privacy_handler.transformIterable([html], "utf-8"), [result])

        html = '<iframe class="myclass" src="http://host.com" width="2" height="4">'
        result = privacy_handler.transformUnicode(html, "utf-8")
        self.assertIn('class="myclass gdpr-iframe"', result)

        html = '<iframe src="http://host.com"><iframe src="http://host2.com">'
        result = privacy_handler.transformUnicode(html, "utf-8")
        self.assertEqual(result.count("This feature requires cookies acceptation"), 2)

    def test_views(self):
        view = queryMultiAdapter((self.portal, self.request), name="allow_iframes")
        self.assertEqual(view(), "false")
        expiration_seconds = time.time() + (60 * 60 * 24 * 365)
        expires = formatdate(expiration_seconds, usegmt=True)
        self.request.response.setCookie(
            "dataprotection", "show_genetic_embed|1", path="/", expires=expires
        )
        self.assertEqual(view(), "true")

    def test_form(self):
        browser = Browser(self.layer["app"])
        browser.open("{}/@@consent".format(self.portal.absolute_url()))
        content = browser.contents
        self.assertIn("Choose to opt in or out of cookies use.", content)



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.privacy.utils import get_all_consent_reasons
from plone import api
from plone.api.exc import CannotGetPortalError
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISiteSchema
from Products.Five import BrowserView
from zope.component import getUtility

import json


class PrivacyView(BrowserView):
    """ """

    def get_analytics(self):
        try:
            portal_privacy = api.portal.get_tool("portal_privacy")
        except CannotGetPortalError:
            return ""
        if not portal_privacy.processingIsAllowed("basic_analytics"):
            return ""
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        try:
            return site_settings.webstats_js or ""
        except AttributeError:
            return ""

    def allow_iframes(self):
        self.request.response.setHeader("Content-type", "application/json")
        portal_privacy = api.portal.get_tool("portal_privacy")
        return json.dumps(portal_privacy.processingIsAllowed("show_genetic_embed"))

    def accept_or_refuse_all(self):
        form = self.request.form
        came_from = form.get("came_from")
        if not came_from:
            came_from = api.portal.get_navigation_root(self.context).absolute_url()
        accept_all = True if "consent" in form else False
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            if accept_all:
                privacy_tool.consentToProcessing(reason.__name__)
            else:
                privacy_tool.objectToProcessing(reason.__name__)
        self.request.response.redirect(came_from)
        return ""



# -*- coding: utf-8 -*-

from imio.smartweb.common.rest.utils import get_restapi_query_lang
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_ACCEPTANCE_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestREST(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_ACCEPTANCE_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})

        self.doc1 = api.content.create(
            container=self.portal,
            type="Document",
            title="Doc 1",
        )
        self.doc1.topics = ["agriculture"]
        api.content.transition(self.doc1, "publish")

        self.doc2 = api.content.create(
            container=self.portal,
            type="Document",
            title="Doc 2",
        )
        self.doc2.iam = ["young", "job_seeker"]
        self.doc2.topics = ["agriculture", "entertainment"]
        api.content.transition(self.doc2, "publish")

        self.event = api.content.create(
            container=self.portal,
            type="Event",
            title="Event",
        )
        api.content.transition(self.event, "publish")

        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_restapi_query_lang(self):
        self.assertEqual(get_restapi_query_lang(), "fr")
        self.request.form["translated_in_nl"] = True
        self.assertEqual(get_restapi_query_lang(), "nl")
        self.request.form["translated_in_en"] = True
        self.assertEqual(get_restapi_query_lang(), "fr")

    def test_search_filters(self):
        query = {
            "portal_type": "Document",
            "metadata_fields": "not_existing",
        }
        response = self.api_session.get("/@search-filters", params=query)
        json = response.json()
        self.assertEqual(len(json), 1)

        query = {
            "portal_type": "Document",
            "metadata_fields": "topics",
        }
        response = self.api_session.get("/@search-filters", params=query)
        json = response.json()
        self.assertEqual(len(json), 1)

        query = {
            "portal_type": "Document",
            "metadata_fields": ["iam", "topics"],
        }
        response = self.api_session.get("/@search-filters", params=query)
        json = response.json()
        self.assertEqual(len(json), 2)
        self.assertEqual(
            json["iam"],
            [
                {"title": "Job seeker", "token": "job_seeker"},
                {"title": "Young", "token": "young"},
            ],
        )
        self.assertEqual(
            json["topics"],
            [
                {"title": "Agriculture", "token": "agriculture"},
                {"title": "Entertainment", "token": "entertainment"},
            ],
        )

        query = {
            "portal_type": "Event",
            "metadata_fields": [
                "iam",
                "topics",
                "effective",
                "end",
                "has_leadimage",
                "start",
                "UID",
            ],
        }
        response = self.api_session.get("/@search-filters", params=query)
        json = response.json()
        self.assertNotIn("effective", json)
        self.assertNotIn("end", json)
        self.assertNotIn("has_leadimage", json)
        self.assertNotIn("start", json)
        self.assertNotIn("UID", json)



# -*- coding: utf-8 -*-

from eea.facetednavigation.subtypes.interfaces import IPossibleFacetedNavigable
from zope.globalrequest import getRequest


def configure_faceted(obj, faceted_config_path):
    if not IPossibleFacetedNavigable.providedBy(obj):
        return
    subtyper = obj.restrictedTraverse("@@faceted_subtyper")
    if not subtyper:
        return
    subtyper.enable()
    with open(faceted_config_path, "rb") as faceted_config:
        obj.unrestrictedTraverse("@@faceted_exportimport").import_xml(
            import_file=faceted_config
        )
    request = getRequest()
    request.response.redirect(obj.absolute_url())



# -*- coding: utf-8 -*-

from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "imio.smartweb.common:testing",
            "imio.smartweb.common:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide unwanted products from site-creation and quickinstaller."""
        return [
            "imio.smartweb.common.upgrades",
        ]


def post_install(context):
    """Post install script"""
    acl_users = api.portal.get_tool("acl_users")
    session = acl_users.session
    # don't make cookie persistent across closing the browser
    session.cookie_lifetime = 0
    # set cookie validity to 8 hours
    session.timeout = 8 * 3600


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from imio.smartweb.common.testing import ImioSmartwebCommonTestCase


class TestVocabularies(ImioSmartwebCommonTestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

    def test_topics(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Topics", 17)

    def test_iam(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.IAm", 10)

    def test_countries(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Countries", 240)

    def test_cities(self):
        self.assertVocabularyLen("imio.smartweb.vocabulary.Cities", 898)



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from imio.smartweb.common.testing import ImioSmartwebCommonTestCase
from imio.smartweb.common.viewlets.colophon import ColophonViewlet
from plone.app.testing import login
from plone.app.testing import logout


class TestViewlets(ImioSmartwebCommonTestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]

    def test_colophon(self):
        logout()
        viewlet = ColophonViewlet(self.portal, self.request, None)
        viewlet.update()
        html = viewlet.render()
        self.assertIn("Agent connection", html)

        login(self.portal, "test")
        viewlet.update()
        html = viewlet.render()
        self.assertNotIn("Agent connection", html)



from plone.namedfile.scaling import NavigationRootScaling as BaseNavigationRootScaling
from zope.cachedescriptors.property import Lazy as lazy_property


class NavigationRootScaling(BaseNavigationRootScaling):
    @lazy_property
    def _supports_image_scales_metadata(self):
        return False



# -*- coding: utf-8 -*-

from plone import api

import logging
import os
import requests

logger = logging.getLogger("imio.smartweb.common")


def ban_physicalpath(request, physical_path):
    portal = api.portal.get()
    caching_servers = os.environ.get("CACHING_SERVERS", "").split(" ")
    forwarded_host = request.get("X-Forwarded-Host", "")
    headers = {"Host": forwarded_host}
    len_portal_path = len(portal.getPhysicalPath())
    path = "/".join(physical_path[len_portal_path:])
    for caching_server in caching_servers:
        if not caching_server:
            continue
        if path:
            ban_url = f"http://{caching_server}/{path}"
        else:
            ban_url = f"http://{caching_server}"
        logger.info(
            f"## BAN ## X-Forwarded-Host : {forwarded_host} ## ban_url : {ban_url}"
        )
        requests.request("BAN", ban_url, headers=headers)



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import clean_invisible_char
from plone import api
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.app.textfield.value import IRichTextValue
from plone.app.textfield.value import RichTextValue
from plone.dexterity.utils import iterSchemata
from zope.lifecycleevent.interfaces import IAttributes
from zope.schema import getFields

import DateTime


def reindex_breadcrumb(obj, event):
    if not hasattr(event, "descriptions") or not event.descriptions:
        return

    for d in event.descriptions:
        if not IAttributes.providedBy(d):
            # we do not have fields change description, but maybe a request
            continue
        if d.interface is not IBasic:
            continue
        if "IBasic.title" in d.attributes:
            brains = api.content.find(context=obj)
            for brain in brains:
                content = brain.getObject()
                content.reindexObject(idxs=["breadcrumb"])
            return


def added_content(obj, event):
    for schema in iterSchemata(obj):
        for name, field in getFields(schema).items():
            value = getattr(obj, name)
            if IRichTextValue.providedBy(value):
                str = clean_invisible_char(value.raw)
                setattr(obj, name, RichTextValue(str))
    reindex_breadcrumb(obj, event)


def modified_content(obj, event):
    for schema in iterSchemata(obj):
        for name, field in getFields(schema).items():
            value = getattr(obj, name)
            if IRichTextValue.providedBy(value):
                str = clean_invisible_char(value.raw)
                setattr(obj, name, RichTextValue(str))
    reindex_breadcrumb(obj, event)


def modified_cropping(obj, event):
    now = DateTime.DateTime()
    obj.setModificationDate(now)
    obj.reindexObject(idxs=["modified"])



# Used to limit description field on all content types
DESCRIPTION_MAX_LENGTH = 700

# Used to translated terms from brains metadatas
# Must be extended by other products
VOCABULARIES_MAPPING = {
    "iam": "imio.smartweb.vocabulary.IAm",
    "topics": "imio.smartweb.vocabulary.Topics",
}

# Used for vocabularies that allow explicitly choosen lang translations
TRANSLATED_VOCABULARIES = [
    "imio.smartweb.vocabulary.Countries",
]



# -*- coding: utf-8 -*-

from plone.namedfile.interfaces import IAvailableSizes
from zope.component import getUtility


class BaseCroppingProvider(object):
    def __init__(self, context):
        self.context = context

    def get_scales(self, fieldname, request=None):
        if self.context.portal_type == "Image":
            # imio.smartweb.core override get_scales.
            # so this is only available for authentic sources
            return []
        allowed_sizes = getUtility(IAvailableSizes)()
        scales = list(allowed_sizes.keys())
        if fieldname == "image":
            # scales used for lead image field
            scales.remove("banner")
        return scales



# -*- coding: utf-8 -*-


def get_all_consent_reasons(privacy_tool):
    for reason in privacy_tool.getAllReasons().values():
        if reason.lawful_basis.__name__ == "consent":
            yield reason



# -*- coding: utf-8 -*-

from datetime import date
from plone import api
from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ColophonViewlet(common.ViewletBase):
    index = ViewPageTemplateFile("colophon.pt")

    def update(self):
        super(ColophonViewlet, self).update()
        self.year = date.today().year
        self.is_anonymous = api.user.is_anonymous()
        self.has_gdpr = api.portal.get_registry_record(
            "imio.gdpr.interfaces.IGDPRSettings.is_text_ready", default=False
        )



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IAm(model.Schema):
    model.fieldset("categorization", label=_("Categorization"), fields=["iam"])
    iam = schema.List(
        title=_("I am"),
        description=_(
            "Important! These categories are used to create lists accessible via the navigation menu"
        ),
        value_type=schema.Choice(vocabulary="imio.smartweb.vocabulary.IAm"),
        required=False,
    )
    directives.widget(iam=SelectFieldWidget)



# -*- coding: utf-8 -*-

from zope.globalrequest import getRequest


def get_restapi_query_lang(query=None):
    if query is None:
        request = getRequest()
        query = request.form
    langs = [
        key.split("translated_in_")[1]
        for key in query
        if key.startswith("translated_in_")
    ]
    if not langs or len(langs) > 1:
        # we don't know how to handle multiple languages query
        return "fr"
    return langs[0]



# -*- coding: utf-8 -*-

from eea.facetednavigation.widgets import ViewPageTemplateFile
from eea.facetednavigation.widgets.interfaces import DefaultSchemata as DS
from eea.facetednavigation.widgets.interfaces import ISchema
from eea.facetednavigation.widgets.widget import Widget
from imio.smartweb.locales import SmartwebMessageFactory as _
from z3c.form import field


class DefaultSchemata(DS):
    """Schemata default"""

    fields = field.Fields(ISchema).select("title")


class RelativePathWidget(Widget):
    """Filter on objects from current folder"""

    widget_type = "relative_path"
    widget_label = _("Relative path widget")
    groups = (DefaultSchemata,)

    index = ViewPageTemplateFile("widget.pt")

    def query(self, form):
        """Returns only objects from current folder"""
        current_path = "/".join(self.context.getPhysicalPath())
        query = {"path": {"query": current_path}}
        return query



# -*- coding: utf-8 -*-

from Acquisition import aq_base
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.utils import parent
from Products.CMFPlone.utils import safe_hasattr


@indexer(IDexterityContent)
def description(obj):
    if obj.description is None:
        return ""
    return obj.description.replace("**", "")


@indexer(IDexterityContent)
def breadcrumb(obj):
    if IPloneSiteRoot.providedBy(obj):
        return ""
    titles = [obj.title]
    while not IPloneSiteRoot.providedBy(parent(obj)):
        obj = parent(obj)
        titles.insert(0, obj.title)
    return " » ".join(titles)


@indexer(IDexterityContent)
def has_leadimage(obj):
    if ILeadImage.providedBy(obj) and getattr(obj, "image", False):
        # we have a lead image on the object
        return True
    if safe_hasattr(aq_base(obj), "image", None):
        # we have an image field on the object, ex: for Image content type
        return True
    return False



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ICropping
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone import api
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from zope.component import getMultiAdapter
from zope.event import notify

import time
import unittest


class TestCropping(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
            id="folder",
        )

    def test_cropping_adapter(self):
        adapter = ICropping(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertNotIn("banner", adapter.get_scales("image", self.request))

        image = api.content.create(
            container=self.folder,
            type="Image",
            title="My image",
            id="my-image",
        )
        adapter = ICropping(image, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual([], adapter.get_scales("image", self.request))

    def test_cropping_view(self):
        cropping_view = getMultiAdapter(
            (self.folder, self.request), name="croppingeditor"
        )
        self.assertEqual(len(list(cropping_view._scales("image"))), 11)

    def test_modified_cropping(self):
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_before = brain.ModificationDate
        time.sleep(1)
        notify(CroppingInfoChangedEvent(self.folder))
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_after = brain.ModificationDate
        self.assertNotEqual(dt_before, dt_after)

        brain = api.content.find(UID=self.folder.UID())[0]
        dt_before = brain.ModificationDate
        time.sleep(1)
        notify(CroppingInfoRemovedEvent(self.folder))
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_after = brain.ModificationDate
        self.assertNotEqual(dt_before, dt_after)



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ILocalManagerAware
from imio.smartweb.common.sharing import permissions
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.workflow.interfaces import ISharingPageRole
from zope.interface import implementer


@implementer(ISharingPageRole)
class LocalManagerRole(object):
    title = _("Can manage locally")
    required_permission = permissions.DelegateLocalManagerRole
    required_interface = ILocalManagerAware



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser

import transaction
import unittest


class TestForms(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_leadimage_caption_field(self):
        folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
        )
        self.check_leadimage_caption_field(folder, container=self.portal)
        document = api.content.create(
            container=folder,
            type="Document",
            title="Document",
        )
        self.check_leadimage_caption_field(document, container=folder)

    def check_leadimage_caption_field(self, obj, container):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(obj.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        lead_image_caption_widget = soup.find(
            id="form-widgets-ILeadImageBehavior-image_caption"
        )
        self.assertIsNotNone(lead_image_caption_widget)
        self.assertEqual(len(lead_image_caption_widget), 0)
        self.assertEqual(lead_image_caption_widget["type"], "hidden")

        browser.open("{}/++add++{}".format(container.absolute_url(), obj.portal_type))
        content = browser.contents
        soup = BeautifulSoup(content)
        lead_image_caption_widget = soup.find(
            id="form-widgets-ILeadImageBehavior-image_caption"
        )
        self.assertIsNotNone(lead_image_caption_widget)
        self.assertEqual(len(lead_image_caption_widget), 0)
        self.assertEqual(lead_image_caption_widget["type"], "hidden")



# -*- coding: utf-8 -*-

from DateTime import DateTime
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestPermissions(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.catalog = self.portal.portal_catalog
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal, type="Folder", id="folder"
        )
        self.doc = api.content.create(container=self.folder, type="Document", id="doc")

    def assertResultsWithRole(self, role, results_nb):
        setRoles(self.portal, TEST_USER_ID, [role])
        brains = self.catalog.searchResults(id="doc")
        self.assertEqual(len(brains), results_nb)

    def test_search_results_expired(self):
        self.assertResultsWithRole("Member", 1)
        self.doc.setExpirationDate(DateTime(2000, 12, 31))
        self.doc.reindexObject()
        self.assertResultsWithRole("Member", 0)
        self.assertResultsWithRole("Contributor", 0)
        self.assertResultsWithRole("Editor", 1)
        self.assertResultsWithRole("Reader", 1)
        self.assertResultsWithRole("Reviewer", 1)
        self.assertResultsWithRole("Manager", 1)
        self.assertResultsWithRole("Site Administrator", 1)



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from plone import api
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.uuid.interfaces import IUUID
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import modified

import os
import unittest


class TestIndexes(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="My Root Folder",
        )

    def test_subjects_in_searchable_text(self):
        catalog = api.portal.get_tool("portal_catalog")
        uuid = IUUID(self.folder)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertNotIn("foo", indexes.get("SearchableText"))
        self.folder.setSubject(["foo"])
        self.folder.reindexObject(idxs=["SearchableText"])
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertIn("foo", indexes.get("SearchableText"))

    def test_breadcrumb(self):
        catalog = api.portal.get_tool("portal_catalog")
        folder = api.content.create(
            container=self.folder,
            type="Folder",
            title="My Sub Folder",
        )
        uuid = IUUID(folder)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("breadcrumb"), "My Root Folder » My Sub Folder")

        sub_folder = api.content.create(
            container=folder,
            type="Folder",
            title="My Sub Sub Folder",
        )
        uuid = IUUID(sub_folder)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("breadcrumb"),
            "My Root Folder » My Sub Folder » My Sub Sub Folder",
        )

        folder.title = "My New Sub Folder"
        modified(folder, Attributes(IBasic, "IBasic.title"))
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("breadcrumb"),
            "My Root Folder » My New Sub Folder » My Sub Sub Folder",
        )

        self.folder.title = "My New Root Folder"
        modified(self.folder, Attributes(IBasic, "IBasic.title"))
        api.content.move(sub_folder, self.folder)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("breadcrumb"), "My New Root Folder » My Sub Sub Folder"
        )

    def test_has_leadimage(self):
        catalog = api.portal.get_tool("portal_catalog")
        uuid = IUUID(self.folder)
        self.assertEqual(len(api.content.find(has_leadimage=True)), 0)
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(brain.has_leadimage, False)
        test_image = os.path.join(os.path.dirname(__file__), "resources/image.png")
        with open(test_image, "rb") as fd:
            self.folder.image = NamedBlobImage(data=fd.read(), filename=test_image)
        self.folder.reindexObject()
        self.assertEqual(len(api.content.find(has_leadimage=True)), 1)
        brain = api.content.find(UID=uuid)[0]
        self.assertEqual(brain.has_leadimage, True)

        collection = api.content.create(
            container=self.portal,
            type="Collection",
            title="My Collection without lead image behavior",
        )
        collection.reindexObject()
        brain = api.content.find(UID=collection.UID())[0]
        self.assertEqual(brain.has_leadimage, False)
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("has_leadimage"), False)
        self.assertEqual(len(api.content.find(has_leadimage=True)), 1)

        image = api.content.create(
            container=self.portal,
            type="Image",
            title="My image",
        )
        with open(test_image, "rb") as fd:
            image.image = NamedBlobImage(data=fd.read(), filename=test_image)
        image.reindexObject()
        brain = api.content.find(UID=image.UID())[0]
        self.assertEqual(brain.has_leadimage, True)
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("has_leadimage"), True)
        self.assertEqual(len(api.content.find(has_leadimage=True)), 2)



# -*- coding: utf-8 -*-

from collective.privacy.interfaces import ICollectivePrivacyLayer
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import schema
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.supermodel import model
from plone.theme.interfaces import IDefaultPloneLayer
from zope.interface import Interface


class IImioSmartwebCommonLayer(
    IDefaultPloneLayer, IPloneFormLayer, ICollectivePrivacyLayer
):
    """Marker interface that defines a browser layer."""


class ILocalManagerAware(Interface):
    """Marker interface that allows a local management."""


class ICropping(Interface):
    def get_scales(fieldname, request=None):
        """Return the available cropping scales for a field on an object."""


class IAddress(model.Schema):
    model.fieldset(
        "address",
        label=_("Address"),
        fields=["street", "number", "complement", "zipcode", "city", "country"],
    )
    street = schema.TextLine(title=_("Street"), required=False)
    number = schema.TextLine(title=_("Number"), required=False)
    complement = schema.TextLine(title=_("Complement"), required=False)
    zipcode = schema.Int(title=_("Zipcode"), required=False)
    city = schema.TextLine(title=_("City"), required=False)
    country = schema.Choice(
        title=_("Country"),
        source="imio.smartweb.vocabulary.Countries",
        required=False,
    )
